#include "encoder.h"
#include "stdint.h"
#include "stm32f4xx_hal.h"


uint8_t prev_state_A;
uint8_t prev_state_B;
uint32_t counter_A = 0;
uint32_t counter_B = 0;
int8_t	encoder_state;

uint8_t io_input_read(uint8_t phase)
{
	return (phase == ENCODER_A) ? HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_7) : HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_9);
}

// Inits states and GPIOs
void encoderInit()
{
	gpioEncoderInit();
	prev_state_A = io_input_read(ENCODER_A);
	prev_state_B = io_input_read(ENCODER_B);
}

// reads the state of an incremental encoder. Reads only falling edge of Phase A
// incremental encoder: stec11B
int8_t encoderRead(int16_t *absolute_value2) // max: -32768 ... +32767
{
	uint8_t state_A;
	uint8_t state_B;
	encoder_state = ENCODER_NO_CHANGE;
	static int8_t absolute_value;

	state_A = io_input_read(ENCODER_A);
	state_B = io_input_read(ENCODER_B);

	if(state_A != prev_state_A && state_A == 1)
	{
		counter_A++;
		if(counter_A >= ENTPRELLEN) // entprellen
		{
			counter_A = ENTPRELLEN; // prevent overflow
			prev_state_A = state_A;
			if(state_A == 1)
			{
				// falling edge
				encoder_state = (state_B == 0) ? ENCODER_CCW : ENCODER_CW;
			}else{
				/* // rising edge
				encoder_state = (state_B == 0) ?  ENCODER_CW : ENCODER_CCW; */
			}
		}
	}
	else if(state_A != prev_state_A && state_A == 0)
	{
		counter_A = 0;
		prev_state_A = state_A;
	}
	else
	{
		counter_A = 0;
	}
	absolute_value += encoder_state;
	*absolute_value2= absolute_value;
	return encoder_state;
}

void gpioEncoderInit()
{

	GPIO_InitTypeDef init;


	/* CLKs enable only if needed */
	__HAL_RCC_GPIOA_CLK_ENABLE();
	__HAL_RCC_GPIOC_CLK_ENABLE();
	__HAL_RCC_GPIOB_CLK_ENABLE();

	//GND
	// Pin7,Port C
	init.Pin = GPIO_PIN_7;
	init.Mode = GPIO_MODE_OUTPUT_PP;
	init.Pull = GPIO_NOPULL;
	init.Speed = GPIO_SPEED_FAST;
	HAL_GPIO_Init(GPIOC, &init);
	HAL_GPIO_WritePin(GPIOC, GPIO_PIN_7, GPIO_PIN_RESET);

	//VCC
	// Pin6,Port B
	init.Pin = GPIO_PIN_6;
	init.Mode = GPIO_MODE_OUTPUT_PP;
	init.Pull = GPIO_NOPULL;
	init.Speed = GPIO_SPEED_FAST;
	HAL_GPIO_Init(GPIOB, &init);
	HAL_GPIO_WritePin(GPIOB, GPIO_PIN_6, GPIO_PIN_SET);

	// Phase A
	// Pin9,Port A
	init.Pin = GPIO_PIN_9;
	init.Mode = GPIO_MODE_INPUT;
	HAL_GPIO_Init(GPIOA, &init);

	// Phase B
	// Pin7,Port A
	init.Pin = GPIO_PIN_7;
	init.Mode = GPIO_MODE_INPUT;
	init.Pull = GPIO_NOPULL;
	init.Speed = GPIO_SPEED_FAST;
	HAL_GPIO_Init(GPIOA, &init);

	// Pin 13, Port C
	init.Pin = GPIO_PIN_13;
	init.Mode = GPIO_MODE_INPUT;
	HAL_GPIO_Init(GPIOC, &init);
}
