//
// This file is part of the GNU ARM Eclipse distribution.
// Copyright (c) 2014 Liviu Ionescu.
//

// ----------------------------------------------------------------------------

#include <stdio.h>
#include "diag/Trace.h"
#include "stm32f411xe.h"
#include "stm32f4xx_hal.h"


/* own includes */
#include "gpio.h"
#include "encoder.h"
#include "timer.h"

void timer2Init();

// ----------------------------------------------------------------------------
//
// STM32F4 empty sample (trace via ITM).
//
// Trace support is enabled by adding the TRACE macro definition.
// By default the trace messages are forwarded to the ITM output,
// but can be rerouted to any device or completely suppressed, by
// changing the definitions required in system/src/diag/trace_impl.c
// (currently OS_USE_TRACE_ITM, OS_USE_TRACE_SEMIHOSTING_DEBUG/_STDOUT).
//

// ----- main() ---------------------------------------------------------------

// Sample pragmas to cope with warnings. Please note the related line at
// the end of this function, used to pop the compiler diagnostics status.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-parameter"
#pragma GCC diagnostic ignored "-Wmissing-declarations"
#pragma GCC diagnostic ignored "-Wreturn-type"

int main(int argc, char* argv[])
{
	/* Variables */
	//int8_t encoder_state = 0;
	gpioInit();
	timer2Init();
	//systickInit();
	//encoderInit();


	// Button: PC13
	// LED: PA5

	// Infinite loop
	while (1)
	{
		/*encoder_state = encoderRead(0);
			if( encoder_state == 1  || encoder_state == -1)
			{
				HAL_GPIO_TogglePin(GPIOA, GPIO_PIN_5);
			}*/
		if((HAL_GetTick() % 100000))
		{
			HAL_GPIO_TogglePin(GPIOA, GPIO_PIN_5);
		}


	}
}

TIM_HandleTypeDef initTimBase;

//TIM2: APB1
// Timer channel 2 as base timer
void timer2Init()
{

	/* enable Clocks */
	__HAL_RCC_TIM2_CLK_ENABLE();

	// every 1ms an overflow
	initTimBase.Instance = TIM2;
	initTimBase.Init.CounterMode = TIM_COUNTERMODE_UP;
	initTimBase.Init.Period = 32;
	initTimBase.Init.Prescaler = 1344;
	initTimBase.Init.ClockDivision = TIM_CLOCKDIVISION_DIV1;
	initTimBase.Init.RepetitionCounter = 0;


	HAL_NVIC_SetPriority(TIM2_IRQn, 4, 1);
	HAL_NVIC_EnableIRQ(TIM2_IRQn);
	//HAL_NVIC_EnableIRQ(SysTick_IRQn);

	HAL_TIM_Base_Init(&initTimBase); 	//MspInit haben keinen Rückgabewert
	HAL_TIM_Base_Start_IT(&initTimBase);


}


void TIM2_IRQHandler(void)
{
	static uint32_t time = 0;
	time ++;

	// 1sek
	if(!(time % 1000))
	{
		HAL_GPIO_TogglePin(GPIOA, GPIO_PIN_5);
	}
	HAL_TIM_IRQHandler(&initTimBase);
}


/*void HAL_SYSTICK_Callback()
{
	static uint32_t clock2 = 0;
	clock2++;
}*/

// just defined interrupt function
/**
 * @brief  Period elapsed callback in non blockfaceing mode
 * @param  htim : TIM handle
 * @retval None
 */
void HAL_TIM_PeriodElapsedCallback(TIM_HandleTypeDef *htim)
{

}

#pragma GCC diagnostic pop

// ----------------------------------------------------------------------------
